// ==========================================================================
// $Source: /home/cvs/Taktuk2/NetworkCore/src/acom_netadmin.h,v $
// Copyright(c)'2001 by Apache Team
// Authors: Jacques Briat, Thierry Gautier, Cyrille Martin
// ==========================================================================
#ifndef _ACOM_NETADMIN_H_
#define _ACOM_NETADMIN_H_

#include <inuk/acom_netbase.h>
#include <inuk/acom_netclient.h>
#include <inuk/acom_netserver.h>
#include <list>
#include <map>

//@Include: acom_netconfig.h acom_comerror.h acom_types.h

// ACOM namespace
namespace ACOM_NET_NAMESPACE {


// ---------------------------------------------------------------------------
// NetworkAdmin class
// ---------------------------------------------------------------------------
/**@name NetworkAdmin
The virtual API for new Inuktitut interface Network.
The Network interface is splitted in serveral sub interfaces.
NetworkServer deals with receiving message.
*/
class NetworkAdmin {
public:
  /** Default Cstor
  */
  NetworkAdmin();
  
  /** Destructor
  */
  virtual ~NetworkAdmin();
  
  /* Projection of LocalNodeNo into the local namespace */
  typedef NetworkBase::LocalNodeNo LocalNodeNo;
  /* Projection of LocalNodeNo into the local namespace */
  typedef NetworkClient::OutChannel OutChannel;
  
  // ---------------------------------------------------------------------------
  // ---------------------------------------------------------------------------
  /** return the connect url address of this node
      @retval string of the url address
  */
  virtual const std::string& get_urlconnect( ) 
       throw (NetworkError);

  // ---------------------------------------------------------------------------
  // ---------------------------------------------------------------------------
  /** plug the current node to a  network address
       @param   nodelabel(IN) network label of remote node
      \warning ajouter les aliasing d'adresse de noeud struct  
      (A voir a partir IP/port, url corba ..) 
  */
  virtual void bind_node( const std::string& nodelabel ="" ) 
       throw (NetworkError);

  /** add a new node to the network
    @param  nodename(IN) network identification of remote node
    @return local node number of the new node
    \warning MUST fail when multiple add of same addreses
    \warning ajouter les aliasing d'adresse de noeud   
    (A voir a partir fd, IP/port, url corba ..)
  */
  virtual void add_node( const std::string& nodelabel, LocalNodeNo  peer ) 
       throw (NetworkError);

   
  /** del a previously added node to the network
      @param  peer  local node Number of remote node to disconnect
      \warning ajouter les aliasing d'adresse de noeud
      ( A voir  partir IP/port, url corba ..)
  */
  virtual void del_node( LocalNodeNo  peer  ) 
      throw (NetworkError);


  // ---------------------------------------------------------------------------    
  // ---------------------------------------------------------------------------    
  /** Return the route (channel) to send data to LocalNodeNo peer
      @param   peer(IN) remote node id
  */
  virtual OutChannel* get_default_route( LocalNodeNo peer ) 
       throw (NetworkError);

  /** Set a channel as a default channel to a destination node
      @param   ch default channel 
  */
  virtual void set_default_route(  OutChannel* ch  ) 
       throw (NetworkError);

  /** Open a channel between local node  to remote peer
      @param   peer(IN) remote node id
      \warning ajouter les aliasing d'adresse de noeud  
  */
  virtual OutChannel* open_channel( LocalNodeNo peer ) 
       throw(ComFailure);
       

  /** Close a channel between local node  to remote peer
      @param   ch (IN)channel to close 
      \warning ajouter les aliasing d'adresse de noeud  
  */
  virtual void close_channel( OutChannel* ch ) 
       throw(ComFailure);


  // ---------------------------------------------------------------------------  
  // node description
  // ---------------------------------------------------------------------------  
  class NodeDesc {
  public:
    /** */
    NodeDesc();

    /** */
    virtual ~NodeDesc();

    /** */
    void init(LocalNodeNo node_rank, const std::string& node_label );

    /**@name Iterator over opened channel
    */
    //@{
    /** */
    typedef std::list<OutChannel*>::iterator       iter_opened_channels; 
    /** */
    typedef std::list<OutChannel*>::const_iterator const_iter_opened_channels; 
    /** */
    iter_opened_channels begin_channels();
    /** */
    iter_opened_channels end_channels();
    /** */
    const_iter_opened_channels begin_channels() const;
    /** */
    const_iter_opened_channels end_channels() const;
    //@}

  public:
    LocalNodeNo            _node_rank;       // - the rank of the peer node in the Network
    std::string            _node_label ;     // - node label
    std::list<OutChannel*> _opened_channels; // - opened channel to this peer
  };
  

  /**@name Iterator over default routes:
  */
  //@{
    /** */
  typedef std::map<LocalNodeNo,OutChannel*>::iterator iter_default_routes; 
    /** */
  typedef std::map<LocalNodeNo,OutChannel*>::const_iterator const_iter_default_routes; 
    /** */
  iter_default_routes begin_routes();
    /** */
  iter_default_routes end_routes();
    /** */
  const_iter_default_routes begin_routes() const;
    /** */
  const_iter_default_routes end_routes() const;
  //@}

  
  /**@name Iterator over known nodes:
  */
  //@{
    /** */
  typedef std::map<LocalNodeNo,NodeDesc*>::iterator       iter_known_nodes; 
    /** */
  //Cyrille 
  typedef std::map<LocalNodeNo,NodeDesc*>::const_iterator const_iter_known_nodes; 
    /** */
  iter_known_nodes begin_known_nodes();
    /** */
  iter_known_nodes end_known_nodes();
    /** */
  const_iter_known_nodes begin_known_nodes() const;
    /** */
  const_iter_known_nodes end_known_nodes() const;
  //@}

protected:
  std::string                        _node_label ;  // local node label
  std::map<LocalNodeNo, NodeDesc*>   _known_nodes;
  std::map<LocalNodeNo, OutChannel*> _default_routes;
}; // -- end class Network


/*
 * inline definition
 *
 */
inline NetworkAdmin::iter_default_routes NetworkAdmin::begin_routes() 
{ return _default_routes.begin(); }

inline NetworkAdmin::iter_default_routes NetworkAdmin::end_routes() 
{ return  _default_routes.end(); }

inline NetworkAdmin::const_iter_default_routes NetworkAdmin::begin_routes() const 
{ return  _default_routes.begin(); }

inline NetworkAdmin::const_iter_default_routes NetworkAdmin::end_routes() const 
{ return  _default_routes.end(); }

inline NetworkAdmin::iter_known_nodes NetworkAdmin::begin_known_nodes() 
{ return _known_nodes.begin(); }

inline NetworkAdmin::iter_known_nodes NetworkAdmin::end_known_nodes() 
{ return  _known_nodes.end(); }

inline NetworkAdmin::const_iter_known_nodes NetworkAdmin::begin_known_nodes() const 
{ return  _known_nodes.begin(); }

inline NetworkAdmin::const_iter_known_nodes NetworkAdmin::end_known_nodes() const 
{ return  _known_nodes.end(); }


// ---------------------------------------------------------------------------  
// node description
// ---------------------------------------------------------------------------  
inline NetworkAdmin::NodeDesc::NodeDesc()
  : _node_rank(0), _node_label ("UNKNOWN_NET"), _opened_channels()
{}

inline NetworkAdmin::NodeDesc::~NodeDesc() {}

inline void NetworkAdmin::NodeDesc::init(LocalNodeNo node_rank, const std::string& node_label ) 
{ _node_rank=node_rank ;_node_label=node_label; }

inline NetworkAdmin::NodeDesc::iter_opened_channels NetworkAdmin::NodeDesc::begin_channels() 
{ return _opened_channels.begin(); }

inline NetworkAdmin::NodeDesc::iter_opened_channels NetworkAdmin::NodeDesc::end_channels() 
{ return  _opened_channels.end(); }

inline NetworkAdmin::NodeDesc::const_iter_opened_channels NetworkAdmin::NodeDesc::begin_channels() const 
{ return  _opened_channels.begin(); }

inline NetworkAdmin::NodeDesc::const_iter_opened_channels NetworkAdmin::NodeDesc::end_channels() const 
{ return  _opened_channels.end(); }


} // end namespace ATH...
#endif 
