// ==========================================================================
// $Source: /home/cvs/Taktuk2/Pthread/src/pth_time.h,v $
// Copyright(c)'2001 by Apache Team
// see the copyright file
// Author: J. Briat & T. Gautier

//
// ==========================================================================
#ifndef _PTH_TIME_H_
#define _PTH_TIME_H_
/*!
\file
\brief Inuktitut INRIA Timer
\author  J. Briat  

This file is dedicated to the time & date classes  Inuktitut INRIA Portable layer ;

 This part is environnement dependant and must be adapted as needed.

\date 2002
*/

#include <sys/time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>   
#include <iostream>
namespace PTH {

/** 
\brief  Physical Time : TR::PhysicalTime

 A pair in second and microsecond

*/
class PhysicalTime{
/// Posix time
  struct timeval _tv; /// Unix case
public :

/// constructor
 PhysicalTime(){_tv.tv_sec=0; _tv.tv_usec=0;};
/// Destructor
~PhysicalTime(){};
/// Equality
 bool operator==(PhysicalTime& t){ 
      return (_tv.tv_sec==t._tv.tv_sec)&&( _tv.tv_usec==t._tv.tv_usec);
 }
/// Not Equality
 bool operator!=(PhysicalTime& t){ 
      return !(_tv.tv_sec==t._tv.tv_sec)&&( _tv.tv_usec==t._tv.tv_usec);
 }
/// this date before date t
 bool operator<(PhysicalTime& t){ 
      return (_tv.tv_sec<t._tv.tv_sec)||
       ((_tv.tv_sec==t._tv.tv_sec)&&( _tv.tv_usec<t._tv.tv_usec));
 }
/// this date before  date t or same date
 bool operator<=(PhysicalTime& t){ 
     return (_tv.tv_sec<t._tv.tv_sec)||
       ((_tv.tv_sec==t._tv.tv_sec)&&( _tv.tv_usec<=t._tv.tv_usec));
   }
/// this date after date t
 bool operator>(PhysicalTime& t){ 
   return (_tv.tv_sec>t._tv.tv_sec)||
     ((_tv.tv_sec==t._tv.tv_sec)&&( _tv.tv_usec>t._tv.tv_usec));
 }
/// this date after  date t or same date
 bool operator>=(PhysicalTime& t){ 
   return (_tv.tv_sec>t._tv.tv_sec)||
       ((_tv.tv_sec==t._tv.tv_sec)&&( _tv.tv_usec>=t._tv.tv_usec));
 }
/// delay between this date and date t
 PhysicalTime operator - (PhysicalTime& t)
   {
     PhysicalTime ret;     
     if (t._tv.tv_usec>_tv.tv_usec)
       {
	 ret._tv.tv_sec=_tv.tv_sec - 1- t._tv.tv_sec;
	 ret._tv.tv_usec=_tv.tv_usec + 1000000 - t._tv.tv_usec;
       }
     else
       {
	 ret._tv.tv_sec=_tv.tv_sec - t._tv.tv_sec;
	 ret._tv.tv_usec=_tv.tv_usec - t._tv.tv_usec;
       }
     return ret;
   }
/// shift this date by delay  t 
 PhysicalTime& operator += (PhysicalTime& t)
   {
     _tv.tv_sec += t._tv.tv_sec;
     _tv.tv_usec += t._tv.tv_usec;
     if (_tv.tv_usec>999999)
       {
	 _tv.tv_usec-=1000000;
	 _tv.tv_sec++;
       }

     return *this;
   }

/// Assign a date value to this date
 PhysicalTime & operator=(const PhysicalTime& t)
   {
     _tv=t._tv;
     return *this;
   }

/// reset yhis date
 PhysicalTime & SetToZero()
   {
     _tv.tv_sec=0;
     _tv.tv_usec=0;
     return *this;
   }

/// change values of date
 PhysicalTime & Setvalues(long int usec,long int sec){
   _tv.tv_sec=sec;
   _tv.tv_usec=usec;
   return *this;
 }

/// print microseconds
 std::ostream & print_usec(std::ostream & out);
/// print seconds 
std::ostream & print_sec(std::ostream & out);
/// get seconds
 int get_sec(){ return _tv.tv_sec;};
/// get microseconds
 int get_usec(){ return _tv.tv_usec;};
friend class PhysicalClock;
};


inline std::ostream & PhysicalTime::print_usec( std::ostream & out)
  { 
    if (_tv.tv_sec!=0)    
      return out <<_tv.tv_sec*1000000+_tv.tv_usec<<"usec";
    else  
      return out <<_tv.tv_usec<<"usec";
  }
inline std::ostream & PhysicalTime::print_sec( std::ostream & out)
  { 
    if (_tv.tv_sec!=0)    
      return out <<_tv.tv_sec<<"."<<_tv.tv_usec<<"(sec)";
    else  
      return out <<"0."<<_tv.tv_usec<<"(sec)";
  } 





/// Output a human readable date
inline std::ostream & operator<<( std::ostream & out, PhysicalTime& t)
{ 
  return out << "Sec("<<(t.get_sec())<<")::Usec("<<(t.get_usec())<<")";
}



/** 
    \brief  Physical Clock : TR::PhysicalClock
    
    
    A clock producing a TR::PhysicalTime
    
*/


 class PhysicalClock{
 public:
/// type of the date
   typedef PhysicalTime Date;
/// constructor
   PhysicalClock(){};
/// destructor
   ~PhysicalClock(){};
/// Get physical time
   PhysicalTime  getDate( ){PhysicalTime d ; ::gettimeofday(&(d._tv),0);return d;}
///  equality of 2 dates 
   bool check( Date t, Date tt){ return t==tt;}
 };
 
 // The host system clock
 //extern PhysicalClock localPhysicalClock;
 
}
#endif
