 // ==========================================================================
// $Source: /home/cvs/Taktuk2/Psocket/src/sock_client.h,v $
// Copyright(c)'2001 by Apache Team
// see the copyright file for more information
// Author: J Briat T.Gautier
// Header file for Inuktitut/Thread/Pthread
//
//
//
// ==========================================================================
#ifndef __SOCK_CLIENT_H_
#define __SOCK_CLIENT_H_

#include <string>
#include <inuk/sock_basic.h>

namespace SOCK {

struct iov_entry {
  void* data;
  int   len;
};

class SocketClient;
class FDSet;


/**
*/
class SocketClient: public Socket {
private:

public:

  /** Constructor
   */
  SocketClient()
   : Socket() 
  {}

  /** Assignement operator : get the default one
   */

  /** connect to server on the associated address
   *  \return zero if connect was succesfull, -1 else
   */
  int connect( );

  /** connect to server on specified address.
   *  \return zero if connect was succesfull, -1 else
   */
  int connect( const Address& );

  /** connect to server on specified host and port
   *  \arg host hostname to connect to
   *  \arg port port to connect on
   *  \return zero if connect was succesfull, -1 else
   */
  int connect(const std::string& host, int port);

  /** write/send data to socket
  *  \arg buf data to be send
  *  \arg size length of bytes to be written
  *  \return number of bytes actually written
  */
  int write(const void* buf, int size);

  /** write/send data to socket
  *  \arg buf data to be send
  *  \arg size length of bytes to be written
  *  \return number of bytes actually written or -1 if and error occurs
  */
  int writen(const void* buf, int size);

  /** write/send data to socket
   *  \arg iov data to be send
   *  \arg count number of bytes to be written
   *  \return -1 if all the data has not been all written
   */
  int writev(const iov_entry* iov, int count);

   /** read/receive data from socket.
  *  \arg iov pointer where data will be stored
  *  \arg count maximum number of bytes that will be read
  *  \return -1 if the number of bytes read is different from size
  */
  int readn(void* buf, int size);

	/** read a line terminated by carrier return
	 *	\arg reference to a string string read will be added to it
	 *	\return -1 if connection closed , else size read 
	 */
	int readline(std::string& buf);

  /** read/receive data from socket.
   *  \arg iov pointer where data will be stored
   *  \arg count maximum number of bytes that will be read
   *  \return number of bytes actually read (may be less than len)
   */
  int read(void* buf, int size);

  /** read/receive data from socket.
   *  \arg buf pointer where data will be stored
   *  \arg size maximum number of bytes that will be read
   *  \return number of bytes actually read (may be less than len)
   */
  int readv(iov_entry* iov, int count);

  /** create a copy of the socket
   * \return the copy
   */
  SocketClient* dup();

  /** create a copy of the socket
   * \arg new_sock  new_sock is the copy of the socket
   */
  void dup2(SocketClient& new_sock);

  /** returns the address of the socket that SOCKET is connected to
   * \return ????
   */
  int  getPeerName (Address& addr) const;

  /** Same a those in the namespace
   */
  static int sock_pair(SocketClient* one, SocketClient* two);
  static int sock_pair(int d, int type, int proto,
                       SocketClient* one, SocketClient* two);

protected:
  friend class FDSet;
};

inline int sock_pair
  (SocketClient* one, SocketClient* two)
{ return SocketClient::sock_pair( one, two); }

inline int sock_pair
  (int d, int type, int proto, SocketClient* one, SocketClient* two)
{ return SocketClient::sock_pair( d, type, proto, one, two); }

}


#endif

