 // ==========================================================================
// $Source: /home/cvs/Taktuk2/Psocket/src/sock_select.h,v $
// Copyright(c)'2001 by Apache Team
// see the copyright file for more information
// Author: J Briat, C. Martin T.Gautier
// Header file for FDSet
// ==========================================================================
/* According to POSIX 1003.1-2001 */

#ifndef __SOCK_SELECT_H_
#define __SOCK_SELECT_H_

#include <inuk/sock_dep.h>
#include <sys/select.h>

/* According to earlier standards */
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#ifdef PTH_USE_PTHREAD_DARWIN
#include <string.h>
#endif


namespace SOCK {

/* fwd declaration */
class SocketClient;

class FDSet {
public:

  /*! Constructor
   */
  FDSet();

  /*! Destructor
   */
  ~FDSet();

  /*! add a given descriptor from a set
   *
   */
  void set(SocketClient* s);

  /*! remove a given descriptor from a set
   */
  void clr(SocketClient* s);

  /*! clear a set
   */
  void zero();

  /*! tests if the descriptor of a socket is part of the set
   * \return nonzero value if the descriptor is part of the set, and zero otherwise
   */
   int isready(SocketClient* s);

  /** Copy the FD set 
  */
  FDSet& operator=( const FDSet& fd )
  { 
#ifdef SOCK_USE_DARWIN
    FD_COPY(&fd._set, &_set); 
#else
    _set = fd._set;
#endif
    _fdmax = fd._fdmax; 
    return *this; 
  }
  
protected:
  int     _fdmax;
  fd_set  _set;
  friend int SOCK::select( FDSet& readset, FDSet& writeset, FDSet& exeptset);
  friend int SOCK::select( FDSet& readset, FDSet& writeset, FDSet& exeptset,
                            struct timeval* Tout);
};


class SocketClient;

/** wait for a number of FDSet to change status
* \return the number of FDSet before the timer expires. On error, -1 is returned
*/
int select ( FDSet& readset, FDSet& writeset, FDSet& exeptset );

/** wait for a number of FDSet to change status
* \return the number of FDSet before the timer expires. On error, -1 is returned
*/
int select ( FDSet& readset, FDSet& writeset, FDSet& exeptset,
                    struct timeval* Tout);


} // namespace SOCK

#endif

