/* Cyrille Martin */
/* laboratoire ID-IMAG, BULL*/
/* Projet LIPS action DYADE*/
/* 07/06/2001 */

#ifndef __CORETCP_H
#define __CORETCP_H



#ifndef MAXBUF
#define MAXBUF 64000
#endif

#ifndef MAXHEADERBUF
#define MAXHEADERBUF 2048
#endif


#include <inuk/pth>
#include <inuk/acom>
#include <inuk/sock>
#include <vector>
#include <pthread.h>
#include <inuk/taktuk/debuglog.h>
#include <inuk/taktuk/panichandler.h>

class routertcp;

class InfoTCPnet
{
 public:
  InfoTCPnet(){rank=0;count_nodes=1;all_links_mut.init();}
  InfoTCPnet(int rank, int count_nodes){this->rank=rank; this->count_nodes=count_nodes;all_links_mut.init();}
  ~InfoTCPnet(){dbg.log("info","DESTRUCTOR\n");}
  
  class link
    {
    public:
      enum {critical,normal,free};
      link(){_sock=NULL;_flag=normal; _mut=NULL;_peer=~acom::Network::LocalNodeNo(0);}
		  link(int f,int p){_sock=NULL;_flag=f; _mut=NULL;_peer=p;}
		  link(SOCK::SocketClient* s,PTH::Mutex* m,int f,int p){_sock=s;_flag=f; _mut=m;_peer=p;}

      ~link(){
        //        if (_mut) _mut->destroy();
        dbg.log("info","link DESTRUCTOR\n");
      }

      SOCK::SocketClient* _sock;
      int _flag;
      acom::Network::LocalNodeNo _peer;
      PTH::Mutex* _mut;
			void lock()
			{
				if(_mut!=NULL)
					_mut->lock();
				else
					dbg.log("info","NULL MUTEX CAN'T LOCK IT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
			}
			void unlock()
			{
				if(_mut!=NULL)
					_mut->unlock();
				else
					dbg.log("info","NULL MUTEX CAN'T UNLOCK IT !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
			}
			bool open()
			{
				return (_sock!=0&& _sock->valid());
			}
			void close()
			{
				_mut->lock();
				_sock->close();
				_mut->unlock();
			}
			void init(SOCK::SocketClient* s,acom::Network::LocalNodeNo p)
			{
				_sock=s;
				_peer=p;
			}
    };
	static link* AllocateLink()
	{
		link* l=NULL;
		l=new link();
		l->_mut=new PTH::Mutex();
		dbg.log("alloclink","mutex value = 0x%X\n", l->_mut);
		l->_mut->init();
		return l;
	}
 
  int env_export();
  int env_recovery();
  acom::Network::LocalNodeNo rank;
  static const acom::Network::LocalNodeNo ALL= ~acom::Network::LocalNodeNo(0);
  static const acom::Network::LocalNodeNo YOU= ~acom::Network::LocalNodeNo(1);
  acom::Network::LocalNodeNo count_nodes;
  std::vector<link*> all_links; // in 0..number of connections used by the router
	PTH::Mutex all_links_mut;
  //see size() on the vector for the number of connections
  std::vector<int> route_table; //in 0 count_nodes-1 give the entry in all_links to reach node X
  friend class routertcp;
};

class routertcp
{
 public:
  /**
     constructor
     @param info this struct define router's parameters cf InfoTCPnet struct
  */
  routertcp();
  ~routertcp();

  int initialize();


  void LoadInfo( InfoTCPnet& info);

  void RefreshInfo(InfoTCPnet& info);
  
  void SetHandler(PanicHandler& h);

  int commit();

  /**
     start the router with the default conf info
  */
  void NET_on();
  /**
     stop the router
  */
  int NET_off();
  /**
     kill the router stop with no waiting any synchro
  */
  void kill();
  /**
     Terminate: the router wait for all connections flaged with "waitfor" closed
  */
  void terminate();
  //void shutdown();

  /**
     function called by the thread
  */
  void routing();

 protected:

  InfoTCPnet info;
  acom::Network* _net;

  void* receive_buffer;
  void* header_buffer;
  // current state of the router thread
  int state;
  bool thread_exist;

  PTH::Mutex state_mutex;

  //semaphore
  PTH::CountDown join;
  //the routing thread
  PTH::Thread routing_thread;
  PTH::ThreadId routing_threadId;
  //pthread_t routing_thread;
  int once_control;

  //to wakup the select of thread_router
  SOCK::SocketClient wakeupW;
  SOCK::SocketClient wakeupR;
	Profile SignalTime;

  //!function which route messages
  friend void thread_router(routertcp* router);


  //!mutex to stop/start the router thread
  /*pthread_mutex_t mutex_waiting;
    pthread_mutex_t mutex_stop;       */
  PTH::Mutex mutex_waiting;
  PTH::Mutex mutex_stop;
  /*pthread_cond_t  cond_waiting;
    pthread_cond_t  cond_stop;    */
  PTH::Condition cond_waiting;
  PTH::Condition cond_stop;

  //! different state of the router
 public:
  static const short running=1;
  static const short stopped=2;
  static const short stopping=3;
  static const short standby=4;
  static const short alone=5;
  static const short killed=6;
  //! broadcast constant
  static const int all=-1;

 private:
  void wait_run();
  void wait_stop();
  void waiting_to_restart();
  void set_router_state(short state);


  //!warning blocking function
  int read_header(SOCK::SocketClient sock, acom::Network::Header* head);
  virtual int process(InfoTCPnet::link* l,int localID)=0; 
  
  int check_router_state();
  int readall(int fd,void* buf,int size);
  int sendall(int fd, void* buf, int size);
  PanicHandler _handler;
};//groutertcp



#endif /*CORETCP_H*/
