/* Cyrille Martin */
/* laboratoire ID-IMAG, BULL*/
/* Projet LIPS action DYADE*/
/* 07/06/2001 */

#ifndef __CONNECTOR_H
#define __CONNECTOR_H

#include <vector>
#include <string>
#include <map>
#include <inuk/pth>
#include <inuk/taktuk/topology.h>
#include <inuk/taktuk/synch_vector.h>
//#include <pthread.h>

#define CHAR_CTRL ''
//bachward declaration
class scheduler;

class run_time
{
	public:
		
		run_time()
		{
			pthread_mutex_init(&_mut,NULL);
			pthread_cond_init(&_cond, NULL);
		};
		~run_time()
		{
			pthread_mutex_destroy(&_mut);
			pthread_cond_destroy(&_cond);
		};

		struct job_time{
			job_time(Profile & proftime, int i){proftime.get_B(&_begin);proftime.get_E(&_end); _jobid=i;};
			job_time(){};
			double time(){
				return ((double)( _end.tv_sec - _begin.tv_sec)+0.000001*(double)(_end.tv_usec - _begin.tv_usec));
			};
			timeval _begin; 
			timeval _end;
			int _jobid;
		};
		
		void set_time(Profile & proftime, int id)
		{
			pthread_mutex_lock(&_mut);
			tab_time.push_back(job_time(proftime,id));
			pthread_cond_signal(&_cond);
			pthread_mutex_unlock(&_mut);
		};
		
		job_time * get_time()
		{
			job_time* jt;
      
			pthread_mutex_lock(&_mut);
			while(tab_time.empty())
			{	
				pthread_cond_wait(&_cond, &_mut);
			}
			jt=new job_time(tab_time.front());
			tab_time.pop_front();
      pthread_mutex_unlock(&_mut);
			return jt;
	
		};

	private:
		std::list<job_time> tab_time;
		pthread_mutex_t _mut; 
		pthread_cond_t _cond;
};

class connector
{
  
 public:
	connector(){
		PTH::Pth::initialize();
		PTH::Pth::commit();
	}
	virtual ~connector(){
		//PTH::Pth::terminate();
	}
  /**
     virtual function to connect this node to other nodes
    
   */
  virtual void run()=0;
	virtual void cancel(){;/*nothing by default*/};

  int connect(synch_vector<topology::host*>*reject_hosts, std::string l_user,PTH::CountDown* cd,scheduler* schd,int id,run_time* rt);
  //int connect(std::vector<topology::host*>*reject_hosts, std::string l_user);

  std::vector<topology::host*>remote_hosts;
  synch_vector<topology::host*>* reject;
  std::string local_user;
	scheduler* _schd;
  void getalive();

  friend void runnable(connector* con); 
  PTH::Thread connect_thread;
  //pthread_t connect_thread;
  PTH::CountDown* _cd;
	int _id;
  run_time* _rt;


};



class connector_collector
{
 public:
  connector_collector();
  ~connector_collector();
  /**
     add a connector object in the connector_collector
     @param id: connector's name 
     @param con: reference to this connector 
   */
	template<class T>
  void add(const std::string& id)
	{
		collector[id]=new WrapperFactory<T>;
	}
  /**
     get a reference to the connector id
   */
  connector* get(const std::string& id)
	{
		if(collector.find(id)!=collector.end())
			return collector[id]->allocate();
		else
			return NULL;
	}

  struct ltstr
  {
    bool operator()(std::string s1, std::string s2) const
    {
      return s1 < s2;
    }
  };
 protected:
	class WrapperFactoryBase{
		public:
		virtual connector* allocate()=0;
	};
	template<class T>
	class WrapperFactory : public WrapperFactoryBase
	{
		public:
		connector* allocate()
		{
			return (connector*)new T;
		}
	};
  /**
     hash table containing all availlable connectors
   */
  std::map<std::string,WrapperFactoryBase*,ltstr> collector;
};


#endif //__CONNECTOR_H
