/* Cyrille Martin */
/* laboratoire ID-IMAG, BULL*/
/* Projet LIPS action DYADE*/
/* 07/06/2001 */



#ifndef __SPREAD_SCHEDULER_H
#define __SPREAD_SCHEDULER_H

#include <inuk/pth>
#include <vector>
#include <deque>
#include <inuk/taktuk/taktuknetwork.h>
#include <inuk/taktuk/topology.h>
#include <inuk/acom>
#include <scheduler_services.h>



template <class T>
class FIFOlist : public std::deque<T>
{
	public:
		FIFOlist(){}
		virtual ~FIFOlist(){}
		void put(T& toto)
		{
			push_back(toto);
		}
		T pop()
		{
			T res;
			res=front();
			pop_front();
			return res;
		}
};


//class scheduler : private PTH::ThreadModel<scheduler>
class scheduler
{
	private:
	public:
		scheduler()
		{
			PTH::Pth::initialize();
			PTH::Pth::commit();
			childs_map_mut.init();
			StealRequestList_mut.init();
			StealRequestList_cond_mut.init();
			StealRequestList_cond.init();
			ToDoList_mut.init();
			ToDoList_cond_mut.init();
			ToDoList_cond.init();
			nbInfoReq_mut.init();
			nbInfoReq_cond.init();
			nbInfoReq=0;
			join.init(1);
			infochilds.init(0);
			spawncompleted.init(1);
			InfoReply.init(1);
			_dojob.initialize(this);
			_steal.initialize(this);
			_setinfo.initialize(this);
			_getinfo.initialize(this);
			_originalhostmap=NULL;
			_maxthread=250;
			info=NULL;
			treesize=1;
			nojob=false;
			End=false;
			acom::Service::Manager.bind(acom::ServiceManager::TAKTUK_LNS,4, (acom::Service*)&_dojob);
			acom::Service::Manager.bind(acom::ServiceManager::TAKTUK_LNS,5, (acom::Service*)&_steal);
			acom::Service::Manager.bind(acom::ServiceManager::TAKTUK_LNS,6, (acom::Service*)&_setinfo);
			acom::Service::Manager.bind(acom::ServiceManager::TAKTUK_LNS,7, (acom::Service*)&_getinfo);
		}
		~scheduler()
		{
			dbg.log("","SCHEDULER DESTROY\n");
			childs_map_mut.destroy();
			StealRequestList_mut.destroy();
			StealRequestList_cond_mut.destroy();
			StealRequestList_cond.destroy();
			ToDoList_mut.destroy();
			ToDoList_cond_mut.destroy();
			ToDoList_cond.destroy();
			nbInfoReq_mut.destroy();
			nbInfoReq_cond.destroy();
            spawncompleted.destroy();
            InfoReply.destroy();
            infochilds.destroy();
            join.destroy();
		}

		void startscheduler();

		/** initialize : init the to do list with hosts defined in hlist
		 *	@param hlist reference to a stl's vector of pointer on host structure
		 *	@param hlistsize add the hlistsize first host included in hlist to the
		 *	to do list
		 *	@param InfoTCPnet structure to reload the router with new links
		 *	@param Taktuk::Network * a Taktuk network initialized this node must
		 *	can communicate with his up node (LocalChannels[0] is initialized
		 *	correctly)
		 */ 
		void initialize(std::vector<topology::host*>* hlist, int hlistsize, InfoTCPnet* i,int maxthread, Taktuk::Network* net);

		/** terminate
		 * 	after this function we known the launch is terminated
		 * 	and all necessary information are hnown by the node
		 * 	- rank
		 * 	- up rank
		 * 	- count node
		 * 	- route table
		 *	when launch sequence is completed this function is unblocked
		 */ 
		void terminate();
		

		/**	get_job function to obtain job -> vector of host* structure CF
		 * topology.h
		 *
		 * @param reference to a vector
		 * new element will be push_backed by this function in it
		 * This function is synchonized
		 * @param integer number of nodes requested
		 * @return size of generated hlist
		 *
		 * If no more job in the local list this function send a request for new
		 * job to his father
		 * wait for it 
		 * 2 cases:
		 * - get job
		 * - no more job on his father
		 *   so it receive an info message with his rank and other global info
		 * 
		 * WARNING it's a blocking function!!
		 * and can be block for a long time may be when all the global job will
		 * be completed
		 *
		 * This function is called by the spawner and the steal service
		 */ 
		int get_job(std::vector<topology::host*>& hlist, int _stealnum);

		/** add_childs add a list of hosts connected to this node
		 * this function is called by the spawner after completed a job list
		 * @param hlist reference to a stl's vector of pointer on host sturcture
		 * after this call new nodes are added in childs_map, info->all_links and
		 * in _net->LocalChannel
		 * New nodes can be reach by the network and can communicate with us
		 */ 
		void add_childs(std::vector<topology::host*>& hlist);

		/** run Thread function
		 * Sckeduler have a thread to record steal supply when there is job in the
		 * ToDoList it send a message to the node supplying
		 * When there is no more Job in the ToDoList he supply job to the up node
		 * if up node reply "no more job", this thread compute and record node
		 * info and childs info (rank, count_nodes, route_table etc...)
		 * and he reply to all this supllying childs their infos
		 */ 
		void run();

/*		inline int getstealnum()
		{
			return 1;
		}
*/
	private:
		/*____________________DATA_________________________________*/


		// scheduler thread
		PTH::Thread sched_thread;

		// to join the scheduler thread
		PTH::CountDown join;
		/** list of all links created by this node. normally all field of this host
		 *  structure are valid. Size : [0..N-1] N number of childs
		 *  can be empty
		 */  
		std::vector<topology::host*> childs_map;
		// to wait for spawner 
		PTH::CountDown spawncompleted;

		// to wait for childs info request
		PTH::Mutex nbInfoReq_mut;
		PTH::Condition nbInfoReq_cond;
		int nbInfoReq;

		//to wait father Info reply
		PTH::CountDown InfoReply;
	
		// to wait for scheduler thread
		PTH::CountDown infochilds;
		
		// attached list mutex
		PTH::Mutex childs_map_mut;
	protected:
		/** ToDoList list of nodes can be steal by the local node or by any childs
		 * Size : [0..N-1] N number of job to do
		 * Launch is completed when this list is empty and when we have received a
		 * "no more job" message
		 * can be empty
		 */ 
		FIFOlist<topology::host*> ToDoList;
	private:
		// attached list mutex
		PTH::Mutex ToDoList_mut;
		PTH::Mutex ToDoList_cond_mut;
		PTH::Condition ToDoList_cond;
		bool nojob;
		
		//reference to the original map to do the update at the terminate
		std::vector<topology::host*>* _originalhostmap;
		/** StealRequestList list of local link which request for job
		 * this list is spooled by the scheduler thread and build by the steal
		 * service
		 * Size : [0..N-1] N number of child
		 * elt <int> : include in [0..X-1] X number of childs
		 * can be empty
		 */
		FIFOlist<int> StealRequestList;
		// attached list mutex & cond
		PTH::Mutex StealRequestList_mut;
		PTH::Mutex StealRequestList_cond_mut;
		PTH::Condition StealRequestList_cond;
		bool End;
		
		/** InfoTCPnet structure
		 * this structure contain all currents link spooled by the router
		 * at begining only one link exist -> 0 to the up rank
		 * WARNING if this node is the root of the tree no links exist at begining
		 */ 
		InfoTCPnet* info;

		//size of the tree where this node is root
		acom::Network::LocalNodeNo treesize;
	protected:
		int _maxthread;
	private:
		//Network pointer to communicate
		Taktuk::Network* _net;
		//scheduler services
		dojob _dojob;
		steal _steal;
		setinfo _setinfo;
		getinfo _getinfo;

		friend class dojob;
		friend class steal;
		friend class setinfo;
		friend class getinfo;
		
		/*______________________FUNCTIONS____________________________*/
		
		/** SubscribeToSteal
		 *	function called by the steal service record the src node to the steal
		 *	request list
		 *	@param local_node id of the local link to reply job or info see thread
		 *	scheduler function
		 */ 
		void SubscribeToSteal(int local_node);
		
		/** WaitForStealer
		 *	this function is called be the scheduler thread for waiting for job
		 *	request
		 *	@return a Local node ID in LocalChannel to send it
		 */ 
		int WaitForStealer();
		
		/** AutoReplyStealer
		 * @param int nb of elt to wait for
		 * blocking function, release when it have reply no more job to nb nodes
		 * release immediatly if nb <= 0
		 */ 
		void AutoReplyStealer(int nb);
		
		/** GetChildRank compute childs rank
		 *	it uses:
		 *	- info->rank
		 *	- childs_map[i]->size
		 *	it builds:
		 *	- childs_map[i]->rank
		 *	- info->all_links[i]._peer
		 *	This function is called by the scheduler thread when there is no more
		 *	job to proceed
		 */ 
		void GetChildsRank();

		/** add_job add hlist in the ToDoList and notify the thread maybe waiting for job
		 *  @param hlist reference to a host* vector
		 *  This function is called by the service tojob
		 */ 
		void add_job(std::vector<topology::host*>& hlist);

		/** set_info build the info structure with info received
		 * @param rank the rank for this node
		 * @param uprank the rank for the up node
		 * @param countnodes total nodes count in the tree
		 * This function notify too the scheduler's thread 
		 * it will be called in spite of add_job
		 */ 
		void set_info(int rank,int uprank, int countnodes);
		
		/** GetRouteTable compute the InfoTCPnet route table
		 * it uses:
		 * - info->count_nodes
		 * - info->rank
		 * - treesize
		 * - childsmap->rank
		 * - childsmap->size
		 */ 
		void GetRouteTable();

		///@return childs_map.size() synchronized function
		int getnbchilds();

		/** GetInfo
		 *	wait for childs GetInfo request
		 *	update local info treesize for sample
		 *	Send Info to it's father if it's not the tree root
		 *	wait father's Info if it's not the root
		 */ 
		void GetInfo();

		/** SendChildsInfo
		 * Send Info to all node's childs
		 *
		 */ 
		void SendChildsInfo();

		/** RefreschInfo()
		 * Update and reload the info structure with the real node count and node id
		 *
		 */
		void RefreschInfo(bool force);

};
#endif
